library length_unit;

import 'package:flutter/material.dart';
import 'package:length_unit/convert_utils.dart';

export 'convert_utils.dart';
export 'dynamic_offset.dart';
export 'dynamic_border_radius.dart';
export 'dynamic_edge_insets.dart';
export 'parse_json.dart';

enum LengthUnit {
  px,

  ///logic pixel
  percent,

  ///percent of the parent size
  //ar,

  ///Aspect Ratio, forbid width to be auto from now on
  vh,

  ///percent of the screen width
  vw,

  ///percent of the screen height
  //auto,

  ///no constraint
}

const Map<String, LengthUnit> lengthUnitMap = {
  //"auto": LengthUnit.auto,
  "\%": LengthUnit.percent,
  "px": LengthUnit.px,
  //"ar": LengthUnit.ar,
  "vh": LengthUnit.vh,
  "vw": LengthUnit.vw,
};

LengthUnit? parseLengthUnit(String? unit) {
  if (unit == null || unit.trim().length == 0) {
    return null;
  }
  return lengthUnitMap[unit];
}

class Length {
  final double value;
  final LengthUnit unit;

  const Length(this.value, {this.unit = LengthUnit.px})
      : assert(value >= 0 && value < double.infinity);

  static Length? fromJson(String? string) {
    if (string == null) return null;
    final RegExp regExp = RegExp('[a-zA-Z\%]');
    int index = string.indexOf(regExp);
    if (index != -1) {
      return Length(double.tryParse(string.substring(0, index)) ?? 0,
          unit: parseLengthUnit(string.substring(index)) ?? LengthUnit.px);
    } else {
      return Length(0);
    }
  }

  String toJson() {
    if (unit == LengthUnit.percent) return value.toString() + "\%";
    return value.toString() + unit.toString().substring(11);
  }

  String getUnit() {
    if (unit == LengthUnit.percent) return "%";
    return unit.toString().substring(11);
  }

  String toDisplayString() {
    switch (unit) {
      case LengthUnit.px:
        return value.roundWithPrecision(1).toString() + " px";
      case LengthUnit.percent:
        return value.roundWithPrecision(1).toString() + "%";
      case LengthUnit.vw:
        return value.roundWithPrecision(1).toString() + "% vw";
      case LengthUnit.vh:
        return value.roundWithPrecision(1).toString() + "% vh";
      default:
        return "";
    }
  }

  double toPX({double? constraintSize, Size? screenSize}) {
    switch (unit) {

      ///for those items with intrinsic size (like image and text)
      ///or means no further constraint at this container level
      //case LengthUnit.auto:
      //  return null;
      case LengthUnit.px:
        return value;
      case LengthUnit.percent:
        return value / 100.0 * (constraintSize ?? 1.0);
      case LengthUnit.vw:
        return value / 100.0 * (screenSize?.width ?? 1.0);
      case LengthUnit.vh:
        return value / 100.0 * (screenSize?.height ?? 1.0);
      default:
        return 0;
    }
  }

  Length copyWith({
    double? value,
    LengthUnit? unit,
  }) {
    return Length(
      value ?? this.value,
      unit: unit ?? this.unit,
    );
  }

  static double? newValue(double? px, LengthUnit unit,
      {double? constraintSize, Size? screenSize}) {
    if (px == null) return null;
    switch (unit) {
      //case LengthUnit.auto:
      //  return null;
      case LengthUnit.px:
        return px;
      case LengthUnit.percent:
        return px / (constraintSize ?? 1.0) * 100.0;
      case LengthUnit.vw:
        return px / (screenSize?.width ?? 1.0) * 100.0;
      case LengthUnit.vh:
        return px / (screenSize?.height ?? 1.0) * 100.0;
    }
  }

  @override
  int get hashCode => hashValues(value, unit);

  @override
  bool operator ==(dynamic other) {
    return other is Length && value == other.value && unit == other.unit;
  }
}
